Attribute VB_Name = "modNodeSorter"
'Accessible UML

'LICENCE
'This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.

'"Accessible UML" is copyright 2001-2004 Paul Blenkhorn and Gareth Evans, at UMIST,
'and is released by them under the GNU Public Licence Version 3. A copy is available
'at http://www.gnu.org/licenses/gpl.txt
'The code was written by Alasdair King.

Option Explicit

'This module sorts the order of nodes in a diagram according to the type
'of diagram. For example, in a use case diagram, actors should come before use
'cases.

Private Const TYPE_USECASE As String = "Use Case Diagram"
Private Const TYPE_STATE As String = "State Diagram"
Private Const TYPE_SEQUENCE As String = "Sequence Diagram"

Public Sub SortDiagram(aDiagram As CNode)
'sorts aDiagram's nodes. aDiagram must be a CNode that is also a diagram
    On Error Resume Next
    'Debug.Print "Doing sort for a " & aDiagram.nodeType & " node, id=" & aDiagram.id
    'go through each node in the diagram, sorting the child nodes
    If aDiagram.childNodes.Count > 1 Then
        Call SortNode(aDiagram)
    End If
End Sub

Private Sub SortNode(aNode As CNode)
'sort the child nodes of this node and any children of those child nodes
    On Error Resume Next

    Dim newChildNodes As New Collection ' handles the new child nodes
    Dim nodeIterator As CNode
    Dim swap As Boolean
    Dim i As Long
    Dim swapTemp As CNode
    
    'Debug.Print "Type: " & ModelType(aNode) & " id=" & aNode.id
    
    Select Case ModelType(aNode)
        Case TYPE_SEQUENCE
            'sequence diagram: sort nodes into time and name
            'first sort by time
            swap = True
            While swap
                swap = False
                For i = 1 To aNode.childNodes.Count - 1
                    'If StrComp(aNode.childNodes(i).nodeName, aNode.childNodes(i + 1).nodeName, vbBinaryCompare) = 1 Then
                    If aNode.childNodes(i).nodeTime > aNode.childNodes(i + 1).nodeTime Then
                        swap = True
                        Set swapTemp = aNode.childNodes(i)
                        Call aNode.childNodes.Remove(i)
                        Call aNode.childNodes.add(swapTemp, swapTemp.id, , i)
                    End If
                Next i
            Wend
            'okay, children now in order of time: sort into blocks of the same name
            Call SortCollaborationChildrenByGroup(aNode)
            'finally sort message sending before message receiving
            Call SortMessageSendingBeforeMessageReceiving(aNode)
        Case TYPE_USECASE
            'use case diagram: sort actors before use cases
            'do actors
            For Each nodeIterator In aNode.childNodes
'                Debug.Print "Node: " & nodeIterator.nodeName
                If nodeIterator.nodeType = UML_NODE_TYPE_ACTOR Then
                    Call newChildNodes.add(nodeIterator, nodeIterator.id)
                End If
            Next nodeIterator
            'do use cases
            For Each nodeIterator In aNode.childNodes
                If nodeIterator.nodeType = UML_NODE_TYPE_USECASE Then
                    Call newChildNodes.add(nodeIterator, nodeIterator.id)
                End If
            Next nodeIterator
            'now swap for existing child nodes
            Set aNode.childNodes = newChildNodes
        Case TYPE_STATE
            'state diagram: sort start states first, end states last
            'start states: these are PseudoState type, we assume: this is what
            'we have always observed, that start nodes are exclusively Pseudostates
            'and that pseudostates are exclusively start nodes
            For Each nodeIterator In aNode.childNodes
                'Debug.Print "nodetyp"; nodeIterator.nodeType
                If nodeIterator.isFirstState Then
                    Call newChildNodes.add(nodeIterator, nodeIterator.id)
                End If
            Next nodeIterator
            'not start or end states
            For Each nodeIterator In aNode.childNodes
                If Not nodeIterator.isLastState And Not nodeIterator.isFirstState Then
                    Call newChildNodes.add(nodeIterator, nodeIterator.id)
                End If
            Next nodeIterator
            'end states
            For Each nodeIterator In aNode.childNodes
                If nodeIterator.isLastState Then
                    Call newChildNodes.add(nodeIterator, nodeIterator.id)
                End If
            Next nodeIterator
            'now swap for existing child nodes
            Set aNode.childNodes = newChildNodes
            'okay, so we've sorted the kids of the current node: now we have to check that
            'each of these kids has sorted kids, so we recursively call this function
            For Each nodeIterator In aNode.childNodes
                Call SortNode(nodeIterator)
            Next nodeIterator
    End Select
End Sub

Private Function ModelType(aNode As CNode) As String
'determines the likely type of this model from its constituent parts. This checks
'the children of a node, and tries to find an identifying one. If it does it
'returns the type as a string using the module constants
    On Error Resume Next
    Dim testType As String
    
    If aNode.nodeType = "Collaboration" Then
        ModelType = TYPE_SEQUENCE
    Else
        If aNode.childNodes.Count = 0 Then
            testType = aNode.nodeType
        Else
            testType = aNode.childNodes(1).nodeType
        End If
        Select Case testType
            Case UML_NODE_TYPE_ACTOR
                ModelType = TYPE_USECASE
            Case UML_NODE_TYPE_USECASE
                ModelType = TYPE_USECASE
            Case UML_NODE_TYPE_COMPOSITESTATE
                ModelType = TYPE_STATE
            Case UML_NODE_TYPE_PSEUDOSTATE
                ModelType = TYPE_STATE
            Case UML_NODE_TYPE_FINALSTATE
                ModelType = TYPE_STATE
            Case UML_NODE_TYPE_SIMPLESTATE
                ModelType = TYPE_STATE
        End Select
    End If
End Function

Private Sub SortCollaborationChildrenByGroup(aNode As CNode)
'sorts the aNode children, already sorted by time, into named groups.
    On Error Resume Next
    Dim currentPoint As Integer
    Dim endPoint As Integer
    Dim currentName As String
    Dim foundName As String
    Dim lookingPoint As Integer
    Dim swapTemp As CNode
    
    currentPoint = 1 'where we are up to in the collection to sort
    endPoint = aNode.childNodes.Count ' where we have to get to
    currentName = aNode.childNodes(currentPoint).messageNodeName
    While currentPoint < endPoint
        'go through the collection of items finding every item that has
        'the same name and move it to just past the current item, increasing
        'the current item track each time
        currentName = aNode.childNodes(currentPoint).messageNodeName
        lookingPoint = currentPoint + 1
        While lookingPoint <= endPoint
            foundName = aNode.childNodes(lookingPoint).messageNodeName
            If foundName = currentName Then
                'found a match: need to move this up to currentpoint
                Set swapTemp = aNode.childNodes(lookingPoint)
                Call aNode.childNodes.Remove(lookingPoint)
                Call aNode.childNodes.add(swapTemp, swapTemp.id, , currentPoint)
                lookingPoint = endPoint ' stop looping and start again at currentpoint
            End If
            lookingPoint = lookingPoint + 1
        Wend
        currentPoint = currentPoint + 1
    Wend
End Sub

Private Sub SortMessageSendingBeforeMessageReceiving(aNode As CNode)
'sorts message sending before message receiving nodes iff they are the same
'node and same time. The user should see "message sent to a at time 3" before
'"message received from a at time 3" when examining the messages sent/received
'from/by a.
    Dim i As Integer
    Dim firstNode As CNode
    Dim secondNode As CNode
    
    'only need to make one pass, so no need to bubblesort
    For i = 1 To aNode.childNodes.Count - 1
        Set firstNode = aNode.childNodes(i)
        Set secondNode = aNode.childNodes(i + 1)
        'check if same time
        If firstNode.nodeTime = secondNode.nodeTime Then
            'okay, same time: same name?
            If firstNode.messageNodeName = secondNode.messageNodeName Then
                'okay, same name: do we need to swap them?
                If firstNode.nodeType = "Message recipient" Then
                    'need to swap!
                    Call aNode.childNodes.Remove(i)
                    Call aNode.childNodes.add(firstNode, firstNode.id, , i)
                End If
            End If
        End If
    Next i
End Sub
